@@ -1,7 +1,7 @@
 package Paws::Kinesis::MemoryCaller;
 use 5.008001;
 
-our $VERSION = "0.06";
+our $VERSION = "0.07";
 
 =head1 NAME
 
@@ -9,23 +9,17 @@ Paws::Kinesis::MemoryCaller - A Paws Caller with in-memory Kinesis.
 
 =head1 SYNOPSIS
 
-    use Paws;
-    use Paws::Kinesis::MemoryCaller;
-
     my $kinesis = Paws->service('Kinesis',
         region      => 'N/A',
         caller      => Paws::Kinesis::MemoryCaller->new(),
         credentials => Paws::Credential::Environment->new(),
     );
 
-    # Create a Kinesis stream...
-    $kinesis->CreateStream(%args);
+    # or simply...
 
-    # Get a shard iterator...
-    $kinesis->GetShardIterator(%args);
+    my $kinesis = Paws::Kinesis::MemoryCaller->new_kinesis();
 
-    # Put a record on a stream...
-    $kinesis->PutRecord(%args);
+    # Then use $kinesis as you would normally, for example:
 
     # Put multiple records on a stream...
     $kinesis->PutRecords(%args);
@@ -78,6 +72,9 @@ use Data::UUID;
 use List::AllUtils qw(first_index);
 use MIME::Base64 qw(decode_base64);
 
+use Paws;
+use Paws::Credential::Environment;
+
 use Paws::Kinesis::DescribeStreamOutput;
 use Paws::Kinesis::GetRecordsOutput;
 use Paws::Kinesis::GetShardIteratorOutput;
@@ -94,8 +91,37 @@ use Paws::Kinesis::Shard;
 use Paws::Kinesis::SequenceNumberRange;
 use Paws::Kinesis::StreamDescription;
 
-has store => (is => 'rw', default => sub { +{} });
-has shard_iterator__address => (is => 'rw', default => sub { +{} });
+has store => (is => 'ro', isa => 'HashRef', default => sub { +{} });
+has shard_iterator__address => (
+    is => 'ro',
+    isa => 'HashRef',
+    default => sub { +{} },
+);
+
+=head1 METHODS
+
+=head2 new_kinesis
+
+Shortcut method to create a new Kinesis service instance that uses this caller.
+Equivalent to:
+
+    Paws->service('Kinesis',
+        caller      => Paws::Kinesis::MemoryCaller->new(),
+        credentials => Paws::Credential::Environment->new(),
+        region      => "N/A",
+    );
+
+=cut
+
+sub new_kinesis {
+    my $class = shift;
+
+    return Paws->service('Kinesis',
+        caller      => $class->new(),
+        credentials => Paws::Credential::Environment->new(),
+        region      => "N/A",
+    );
+}
 
 sub caller_to_response {}
 
@@ -327,21 +353,22 @@ sub _get_records {
     my $shard_id = $address->{shard_id};
     my $index = $address->{index};
 
-    my $records = $self->_get_records_from_store($stream_name, $shard_id);
+    my @stream_shard_records =
+        @{$self->_get_records_from_store($stream_name, $shard_id)};
 
-    # make a copy for splice...
-    my @records = @$records;
+    my $end_index = defined $limit
+        ? $index + $limit - 1
+        : scalar @stream_shard_records - 1;
+
+    my $records = [ @stream_shard_records[$index..$end_index] ];
+
+    my $next_shard_iterator = $self->_create_shard_iterator(
+        $stream_name, $shard_id, $end_index + 1 ,
+    );
 
     return Paws::Kinesis::GetRecordsOutput->new(
-        Records => [
-            defined $limit
-                ? splice(@records, $index, $limit)
-                : splice(@records, $index)
-        ],
-        NextShardIterator => $self->_get_shard_iterator_latest(
-            stream_name => $stream_name,
-            shard_id => $shard_id,
-        ),
+        Records => $records,
+        NextShardIterator => $next_shard_iterator,
     );
 }
 
